@@ -1,6 +1,13 @@
+import os
+
 from pyVmomi import vmodl
 
 
+def get_bool_env(key: str, default: bool):
+    value = os.environ.get(key, default)
+    return value if type(value) == bool else value.lower() == 'true'
+
+
 def batch_fetch_properties(content, obj_type, properties):
     view_ref = content.viewManager.CreateContainerView(
         container=content.rootFolder,
