@@ -15,8 +15,9 @@ import ssl
 import sys
 import traceback
 import pytz
+import logging
 
-from yamlconfig import YamlConfig
+import yaml
 
 # Twisted
 from twisted.web.server import Site, NOT_DONE_YET
@@ -31,18 +32,19 @@ from pyVim import connect
 from prometheus_client.core import GaugeMetricFamily
 from prometheus_client import CollectorRegistry, generate_latest
 
-from .helpers import batch_fetch_properties
+from .helpers import batch_fetch_properties, get_bool_env
 from .defer import parallelize, run_once_property
 
 
 class VmwareCollector():
 
-    def __init__(self, host, username, password, collect_only, ignore_ssl=False):
+    def __init__(self, host, username, password, collect_only, specs_size, ignore_ssl=False):
         self.host = host
         self.username = username
         self.password = password
         self.ignore_ssl = ignore_ssl
         self.collect_only = collect_only
+        self.specs_size = int(specs_size)
 
     def _create_metric_containers(self):
         metric_list = {}
@@ -63,6 +65,14 @@ class VmwareCollector():
                 'vmware_vm_memory_max',
                 'VMWare VM Memory Max availability in Mbytes',
                 labels=['vm_name', 'host_name', 'dc_name', 'cluster_name']),
+            'vmware_vm_max_cpu_usage': GaugeMetricFamily(
+                'vmware_vm_max_cpu_usage',
+                'VMWare VM Cpu Max availability in hz',
+                labels=['vm_name', 'host_name', 'dc_name', 'cluster_name']),
+            'vmware_vm_template': GaugeMetricFamily(
+                'vmware_vm_template',
+                'VMWare VM Template (true / false)',
+                labels=['vm_name', 'host_name', 'dc_name', 'cluster_name']),
             }
         metric_list['vmguests'] = {
             'vmware_vm_guest_disk_free': GaugeMetricFamily(
@@ -139,6 +149,10 @@ class VmwareCollector():
                 'vmware_host_power_state',
                 'VMWare Host Power state (On / Off)',
                 labels=['host_name', 'dc_name', 'cluster_name']),
+            'vmware_host_standby_mode': GaugeMetricFamily(
+                'vmware_host_standby_mode',
+                'VMWare Host Standby Mode (entering / exiting / in / none)',
+                labels=['host_name', 'dc_name', 'cluster_name', 'standby_mode_state']),
             'vmware_host_connection_state': GaugeMetricFamily(
                 'vmware_host_connection_state',
                 'VMWare Host connection state (connected / disconnected / notResponding)',
@@ -171,6 +185,14 @@ class VmwareCollector():
                 'vmware_host_memory_max',
                 'VMWare Host Memory Max availability in Mbytes',
                 labels=['host_name', 'dc_name', 'cluster_name']),
+            'vmware_host_product_info': GaugeMetricFamily(
+                'vmware_host_product_info',
+                'A metric with a constant "1" value labeled by version and build from os the host.',
+                labels=['host_name', 'dc_name', 'cluster_name', 'version', 'build']),
+            'vmware_host_hardware_info': GaugeMetricFamily(
+                'vmware_host_hardware_info',
+                'A metric with a constant "1" value labeled by model and cpu model from the host.',
+                labels=['host_name', 'dc_name', 'cluster_name', 'hardware_model', 'hardware_cpu_model']),
             }
 
         metrics = {}
@@ -187,7 +209,7 @@ class VmwareCollector():
 
         metrics = self._create_metric_containers()
 
-        log("Start collecting metrics from {0}".format(vsphere_host))
+        logging.info("Start collecting metrics from {vsphere_host}".format(vsphere_host=vsphere_host))
 
         self._labels = {}
 
@@ -208,12 +230,13 @@ class VmwareCollector():
 
         if collect_only['hosts'] is True:
             tasks.append(self._vmware_get_hosts(metrics))
+            tasks.append(self._vmware_get_host_perf_manager_metrics(metrics))
 
         yield parallelize(*tasks)
 
-        yield self._vmware_disconnect
+        yield self._vmware_disconnect()
 
-        log("Finished collecting metrics from {0}".format(vsphere_host))
+        logging.info("Finished collecting metrics from {vsphere_host}".format(vsphere_host=vsphere_host))
 
         return list(metrics.values())   # noqa: F705
 
@@ -242,19 +265,19 @@ class VmwareCollector():
             return vmware_connect
 
         except vmodl.MethodFault as error:
-            log("Caught vmodl fault: " + error.msg)
+            logging.error("Caught vmodl fault: {error}".format(error=error.msg))
             return None
 
     @run_once_property
     @defer.inlineCallbacks
     def content(self):
-        log("Retrieving service instance content")
+        logging.info("Retrieving service instance content")
         connection = yield self.connection
         content = yield threads.deferToThread(
             connection.RetrieveContent
         )
 
-        log("Retrieved service instance content")
+        logging.info("Retrieved service instance content")
         return content
 
     @defer.inlineCallbacks
@@ -271,7 +294,7 @@ class VmwareCollector():
     @run_once_property
     @defer.inlineCallbacks
     def datastore_inventory(self):
-        log("Fetching vim.Datastore inventory")
+        logging.info("Fetching vim.Datastore inventory")
         start = datetime.datetime.utcnow()
         properties = [
             'name',
@@ -289,13 +312,14 @@ class VmwareCollector():
             vim.Datastore,
             properties
         )
-        log("Fetched vim.Datastore inventory (%s)", datetime.datetime.utcnow() - start)
+        fetch_time = datetime.datetime.utcnow() - start
+        logging.info("Fetched vim.Datastore inventory ({fetch_time})".format(fetch_time=fetch_time))
         return datastores
 
     @run_once_property
     @defer.inlineCallbacks
     def host_system_inventory(self):
-        log("Fetching vim.HostSystem inventory")
+        logging.info("Fetching vim.HostSystem inventory")
         start = datetime.datetime.utcnow()
         properties = [
             'name',
@@ -303,25 +327,31 @@ class VmwareCollector():
             'summary.hardware.numCpuCores',
             'summary.hardware.cpuMhz',
             'summary.hardware.memorySize',
+            'summary.config.product.version',
+            'summary.config.product.build',
             'runtime.powerState',
+            'runtime.standbyMode',
             'runtime.bootTime',
             'runtime.connectionState',
             'runtime.inMaintenanceMode',
             'summary.quickStats.overallCpuUsage',
             'summary.quickStats.overallMemoryUsage',
+            'summary.hardware.cpuModel',
+            'summary.hardware.model',
         ]
 
         host_systems = yield self.batch_fetch_properties(
             vim.HostSystem,
             properties,
         )
-        log("Fetched vim.HostSystem inventory (%s)", datetime.datetime.utcnow() - start)
+        fetch_time = datetime.datetime.utcnow() - start
+        logging.info("Fetched vim.HostSystem inventory ({fetch_time})".format(fetch_time=fetch_time))
         return host_systems
 
     @run_once_property
     @defer.inlineCallbacks
     def vm_inventory(self):
-        log("Fetching vim.VirtualMachine inventory")
+        logging.info("Fetching vim.VirtualMachine inventory")
         start = datetime.datetime.utcnow()
         properties = [
             'name',
@@ -335,6 +365,8 @@ class VmwareCollector():
                 'runtime.bootTime',
                 'summary.config.numCpu',
                 'summary.config.memorySizeMB',
+                'runtime.maxCpuUsage',
+                'summary.config.template',
             ])
 
         if self.collect_only['vmguests'] is True:
@@ -352,7 +384,8 @@ class VmwareCollector():
             vim.VirtualMachine,
             properties,
         )
-        log("Fetched vim.VirtualMachine inventory (%s)", datetime.datetime.utcnow() - start)
+        fetch_time = datetime.datetime.utcnow() - start
+        logging.info("Fetched vim.VirtualMachine inventory ({fetch_time})".format(fetch_time=fetch_time))
         return virtual_machines
 
     @run_once_property
@@ -369,58 +402,66 @@ class VmwareCollector():
     @defer.inlineCallbacks
     def datastore_labels(self):
 
-        def _collect(dc, node):
+        def _collect(node, level=1, dc=None, storagePod=""):
             inventory = {}
-            for folder in node.childEntity:  # Iterate through datastore folders
-                if isinstance(folder, vim.Datastore):  # Unclustered datastore
-                    row = inventory[folder.name] = [
-                        folder.name,
-                        dc.name,
-                    ]
-                    if isinstance(node, vim.StoragePod):
-                        row.append(node.name)
-                    else:
-                        row.append('')
-
-                else:  # Folder is a Datastore Cluster
-                    inventory.update(_collect(dc, folder))
+            if isinstance(node, vim.Folder) and not isinstance(node, vim.StoragePod):
+                logging.debug("[Folder    ] {level} {name}".format(name=node.name, level=('-' * level).ljust(7)))
+                for child in node.childEntity:
+                    inventory.update(_collect(child, level + 1, dc))
+            elif isinstance(node, vim.Datacenter):
+                logging.debug("[Datacenter] {level} {name}".format(name=node.name, level=('-' * level).ljust(7)))
+                inventory.update(_collect(node.datastoreFolder, level + 1, node.name))
+            elif isinstance(node, vim.Folder) and isinstance(node, vim.StoragePod):
+                logging.debug("[StoragePod] {level} {name}".format(name=node.name, level=('-' * level).ljust(7)))
+                for child in node.childEntity:
+                    inventory.update(_collect(child, level + 1, dc, node.name))
+            elif isinstance(node, vim.Datastore):
+                logging.debug("[Datastore ] {level} {name}".format(name=node.name, level=('-' * level).ljust(7)))
+                inventory[node.name] = [node.name, dc, storagePod]
+            else:
+                logging.debug("[?         ] {level} {node}".format(node=node, level=('-' * level).ljust(7)))
             return inventory
 
         labels = {}
         dcs = yield self.datacenter_inventory
         for dc in dcs:
-            result = yield threads.deferToThread(lambda: _collect(dc, dc.datastoreFolder))
+            result = yield threads.deferToThread(lambda: _collect(dc))
             labels.update(result)
-
         return labels
 
     @run_once_property
     @defer.inlineCallbacks
     def host_labels(self):
 
-        def _collect(dc, node):
-            host_inventory = {}
-            for folder in node.childEntity:
-                if hasattr(folder, 'host'):
-                    for host in folder.host:  # Iterate through Hosts in the Cluster
-                        host_name = host.summary.config.name.rstrip('.')
-                        host_inventory[host._moId] = [
-                            host_name,
-                            dc.name,
-                            folder.name if isinstance(folder, vim.ClusterComputeResource) else ''
-                        ]
-
-                if isinstance(folder, vim.Folder):
-                    host_inventory.extend(_collect(dc, folder))
-
-            return host_inventory
+        def _collect(node, level=1, dc=None, folder=None):
+            inventory = {}
+            if isinstance(node, vim.Folder) and not isinstance(node, vim.StoragePod):
+                logging.debug("[Folder    ] {level} {name}".format(level=('-' * level).ljust(7), name=node.name))
+                for child in node.childEntity:
+                    inventory.update(_collect(child, level + 1, dc))
+            elif isinstance(node, vim.Datacenter):
+                logging.debug("[Datacenter] {level} {name}".format(level=('-' * level).ljust(7), name=node.name))
+                inventory.update(_collect(node.hostFolder, level + 1, node.name))
+            elif isinstance(node, vim.ComputeResource):
+                logging.debug("[ComputeRes] {level} {name}".format(level=('-' * level).ljust(7), name=node.name))
+                for host in node.host:
+                    inventory.update(_collect(host, level + 1, dc, node))
+            elif isinstance(node, vim.HostSystem):
+                logging.debug("[HostSystem] {level} {name}".format(level=('-' * level).ljust(7), name=node.name))
+                inventory[node._moId] = [
+                    node.summary.config.name.rstrip('.'),
+                    dc,
+                    folder.name if isinstance(folder, vim.ClusterComputeResource) else ''
+                    ]
+            else:
+                logging.debug("[?         ] {level} {node}".format(level=('-' * level).ljust(7), node=node))
+            return inventory
 
         labels = {}
         dcs = yield self.datacenter_inventory
         for dc in dcs:
-            result = yield threads.deferToThread(lambda: _collect(dc, dc.hostFolder))
+            result = yield threads.deferToThread(lambda: _collect(dc))
             labels.update(result)
-
         return labels
 
     @run_once_property
@@ -430,8 +471,15 @@ class VmwareCollector():
 
         labels = {}
         for moid, row in virtual_machines.items():
-            host_moid = row['runtime.host']._moId
-            labels[moid] = [row['name']] + host_labels[host_moid]
+
+            host_moid = None
+            if 'runtime.host' in row:
+                host_moid = row['runtime.host']._moId
+
+            labels[moid] = [row['name']]
+
+            if host_moid in host_labels:
+                labels[moid] = labels[moid] + host_labels[host_moid]
 
         return labels
 
@@ -482,12 +530,19 @@ class VmwareCollector():
         Get Datastore information
         """
 
-        log("Starting datastore metrics collection")
         results, datastore_labels = yield parallelize(self.datastore_inventory, self.datastore_labels)
 
         for datastore_id, datastore in results.items():
-            name = datastore['name']
-            labels = datastore_labels[name]
+            try:
+                name = datastore['name']
+                labels = datastore_labels[name]
+            except KeyError as e:
+                logging.info(
+                    "Key error, unable to register datastore {error}, datastores are {datastore_labels}".format(
+                        error=e, datastore_labels=datastore_labels
+                    )
+                )
+                continue
 
             ds_capacity = float(datastore.get('summary.capacity', 0))
             ds_freespace = float(datastore.get('summary.freeSpace', 0))
@@ -518,25 +573,40 @@ class VmwareCollector():
                     datastore['summary.accessible'] * 1,
                 )
 
-        log("Finished datastore metrics collection")
+        return results
 
     @defer.inlineCallbacks
     def _vmware_get_vm_perf_manager_metrics(self, vm_metrics):
-        log('START: _vmware_get_vm_perf_manager_metrics')
+        logging.info('START: _vmware_get_vm_perf_manager_metrics')
 
         virtual_machines, counter_info = yield parallelize(self.vm_inventory, self.counter_ids)
 
         # List of performance counter we want
         perf_list = [
             'cpu.ready.summation',
+            'cpu.maxlimited.summation',
             'cpu.usage.average',
             'cpu.usagemhz.average',
+            'cpu.costop.summation',
+            'cpu.idle.summation',
+            'cpu.demand.average',
+            'mem.usage.average',
+            'mem.consumed.average',
+            'mem.active.average',
+            'mem.swapped.average',
+            'mem.vmmemctl.average',
+            'disk.maxTotalLatency.latest',
             'disk.usage.average',
             'disk.read.average',
             'disk.write.average',
-            'mem.usage.average',
             'net.received.average',
             'net.transmitted.average',
+            'net.multicastRx.summation',
+            'net.multicastTx.summation',
+            'net.broadcastTx.summation',
+            'net.broadcastRx.summation',
+            'net.droppedRx.summation',
+            'net.droppedTx.summation',
         ]
 
         # Prepare gauges
@@ -571,30 +641,127 @@ class VmwareCollector():
 
         content = yield self.content
 
-        results, labels = yield parallelize(
-            threads.deferToThread(content.perfManager.QueryStats, querySpec=specs),
-            self.vm_labels,
-        )
-
-        for ent in results:
-            for metric in ent.value:
-                vm_metrics[metric_names[metric.id.counterId]].add_metric(
-                    labels[ent.entity._moId],
-                    float(sum(metric.value)),
+        if len(specs) > 0:
+            chunks = [specs[x:x+self.specs_size] for x in range(0, len(specs), self.specs_size)]
+            for list_specs in chunks:
+                results, labels = yield parallelize(
+                    threads.deferToThread(content.perfManager.QueryStats, querySpec=list_specs),
+                    self.vm_labels,
                 )
-        log('FIN: _vmware_get_vm_perf_manager_metrics')
+
+                for ent in results:
+                    for metric in ent.value:
+                        vm_metrics[metric_names[metric.id.counterId]].add_metric(
+                            labels[ent.entity._moId],
+                            float(sum(metric.value)),
+                        )
+
+        logging.info('FIN: _vmware_get_vm_perf_manager_metrics')
+
+    @defer.inlineCallbacks
+    def _vmware_get_host_perf_manager_metrics(self, host_metrics):
+        logging.info('START: _vmware_get_host_perf_manager_metrics')
+
+        host_systems, counter_info = yield parallelize(self.host_system_inventory, self.counter_ids)
+
+        # List of performance counter we want
+        perf_list = [
+            'cpu.costop.summation',
+            'cpu.demand.average',
+            'cpu.idle.summation',
+            'cpu.ready.summation',
+            'cpu.swapwait.summation',
+            'cpu.usage.average',
+            'cpu.usagemhz.average',
+            'cpu.used.summation',
+            'disk.read.average',
+            'disk.write.average',
+            'mem.active.average',
+            'mem.latency.average',
+            'mem.swapin.average',
+            'mem.swapinRate.average',
+            'mem.swapout.average',
+            'mem.swapoutRate.average',
+            'mem.vmmemctl.average',
+            'net.bytesRx.average',
+            'net.bytesTx.average',
+            'net.droppedRx.summation',
+            'net.droppedTx.summation',
+            'net.errorsRx.summation',
+            'net.errorsTx.summation',
+            'net.usage.average',
+        ]
+
+        # Prepare gauges
+        for p in perf_list:
+            p_metric = 'vmware_host_' + p.replace('.', '_')
+            host_metrics[p_metric] = GaugeMetricFamily(
+                p_metric,
+                p_metric,
+                labels=['host_name', 'dc_name', 'cluster_name'])
+
+        metrics = []
+        metric_names = {}
+        for perf_metric in perf_list:
+            perf_metric_name = 'vmware_host_' + perf_metric.replace('.', '_')
+            counter_key = counter_info[perf_metric]
+            metrics.append(vim.PerformanceManager.MetricId(
+                counterId=counter_key,
+                instance=''
+            ))
+            metric_names[counter_key] = perf_metric_name
+
+        specs = []
+        for host in host_systems.values():
+            if host.get('runtime.powerState') != 'poweredOn':
+                continue
+            specs.append(vim.PerformanceManager.QuerySpec(
+                maxSample=1,
+                entity=host['obj'],
+                metricId=metrics,
+                intervalId=20
+            ))
+
+        content = yield self.content
+
+        if len(specs) > 0:
+            results, labels = yield parallelize(
+                threads.deferToThread(content.perfManager.QueryStats, querySpec=specs),
+                self.host_labels,
+            )
+
+            for ent in results:
+                for metric in ent.value:
+                    host_metrics[metric_names[metric.id.counterId]].add_metric(
+                        labels[ent.entity._moId],
+                        float(sum(metric.value)),
+                     )
+
+        logging.info('FIN: _vmware_get_host_perf_manager_metrics')
 
     @defer.inlineCallbacks
     def _vmware_get_vms(self, metrics):
         """
         Get VM information
         """
-        log("Starting vm metrics collection")
+        logging.info("Starting vm metrics collection")
 
         virtual_machines, vm_labels = yield parallelize(self.vm_inventory, self.vm_labels)
 
         for moid, row in virtual_machines.items():
+            # Ignore vm if field "runtime.host" does not exist
+            # It will happen during a VM is cloning
+            if 'runtime.host' not in row:
+                continue
+
             labels = vm_labels[moid]
+            labels_cnt = len(labels)
+
+            if labels_cnt < 4:
+                logging.info("Only ${cnt}/4 labels (vm, host, dc, cluster) found, filling n/a".format(cnt=labels_cnt))
+
+            for i in range(labels_cnt, 4):
+                labels.append('n/a')
 
             if 'runtime.powerState' in row:
                 power_state = 1 if row['runtime.powerState'] == 'poweredOn' else 0
@@ -612,6 +779,12 @@ class VmwareCollector():
             if 'summary.config.memorySizeMB' in row:
                 metrics['vmware_vm_memory_max'].add_metric(labels, row['summary.config.memorySizeMB'])
 
+            if 'runtime.maxCpuUsage' in row:
+                metrics['vmware_vm_max_cpu_usage'].add_metric(labels, row['runtime.maxCpuUsage'])
+
+            if 'summary.config.template' in row:
+                metrics['vmware_vm_template'].add_metric(labels, row['summary.config.template'])
+
             if 'guest.disk' in row and len(row['guest.disk']) > 0:
                 for disk in row['guest.disk']:
                     metrics['vmware_vm_guest_disk_free'].add_metric(
@@ -650,24 +823,47 @@ class VmwareCollector():
                         snapshot['timestamp_seconds'],
                     )
 
-        log("Finished vm metrics collection")
+        logging.info("Finished vm metrics collection")
 
     @defer.inlineCallbacks
     def _vmware_get_hosts(self, host_metrics):
         """
         Get Host (ESXi) information
         """
-        log("Starting host metrics collection")
+        logging.info("Starting host metrics collection")
 
         results, host_labels = yield parallelize(self.host_system_inventory, self.host_labels)
 
         for host_id, host in results.items():
-            labels = host_labels[host_id]
+            try:
+                labels = host_labels[host_id]
+            except KeyError as e:
+                logging.info(
+                    "Key error, unable to register host {error}, host labels are {host_labels}".format(
+                        error=e, host_labels=host_labels
+                    )
+                )
+                continue
+
+            # Standby Mode
+            standby_mode = 1 if host.get('runtime.standbyMode') == 'in' else 0
+            standby_mode_state = host.get('runtime.standbyMode', 'unknown')
+            host_metrics['vmware_host_standby_mode'].add_metric(
+                labels + [standby_mode_state],
+                standby_mode
+            )
 
             # Power state
             power_state = 1 if host['runtime.powerState'] == 'poweredOn' else 0
             host_metrics['vmware_host_power_state'].add_metric(labels, power_state)
 
+            # Host connection state (connected, disconnected, notResponding)
+            connection_state = host.get('runtime.connectionState', 'unknown')
+            host_metrics['vmware_host_connection_state'].add_metric(
+                labels + [connection_state],
+                1
+            )
+
             if not power_state:
                 continue
 
@@ -679,13 +875,6 @@ class VmwareCollector():
                     self._to_epoch(host['runtime.bootTime'])
                 )
 
-            # Host connection state (connected, disconnected, notResponding)
-            connection_state = host.get('runtime.connectionState', 'unknown')
-            host_metrics['vmware_host_connection_state'].add_metric(
-                labels + [connection_state],
-                1
-            )
-
             # Host in maintenance mode?
             if 'runtime.inMaintenanceMode' in host:
                 host_metrics['vmware_host_maintenance_mode'].add_metric(
@@ -722,7 +911,20 @@ class VmwareCollector():
                     float(host['summary.hardware.memorySize']) / 1024 / 1024
                 )
 
-        log("Finished host metrics collection")
+            config_ver = host.get('summary.config.product.version', 'unknown')
+            build_ver = host.get('summary.config.product.build', 'unknown')
+            host_metrics['vmware_host_product_info'].add_metric(
+                labels + [config_ver, build_ver],
+                1
+            )
+
+            hardware_cpu_model = host.get('summary.hardware.cpuModel', 'unknown')
+            hardware_model = host.get('summary.hardware.model', 'unknown')
+            host_metrics['vmware_host_hardware_info'].add_metric(
+                labels + [hardware_model, hardware_cpu_model],
+                1
+            )
+        logging.info("Finished host metrics collection")
         return results
 
 
@@ -749,9 +951,11 @@ class VMWareMetricsResource(Resource):
     def configure(self, args):
         if args.config_file:
             try:
-                self.config = YamlConfig(args.config_file)
+                with open(args.config_file) as cf:
+                    self.config = yaml.load(cf, Loader=yaml.FullLoader)
+
                 if 'default' not in self.config.keys():
-                    log("Error, you must have a default section in config file (for now)")
+                    logging.error("Error, you must have a default section in config file (for now)")
                     exit(1)
                 return
             except Exception as exception:
@@ -762,13 +966,14 @@ class VMWareMetricsResource(Resource):
                 'vsphere_host': os.environ.get('VSPHERE_HOST'),
                 'vsphere_user': os.environ.get('VSPHERE_USER'),
                 'vsphere_password': os.environ.get('VSPHERE_PASSWORD'),
-                'ignore_ssl': os.environ.get('VSPHERE_IGNORE_SSL', False),
+                'ignore_ssl': get_bool_env('VSPHERE_IGNORE_SSL', False),
+                'specs_size': os.environ.get('VSPHERE_SPECS_SIZE', 5000),
                 'collect_only': {
-                    'vms': os.environ.get('VSPHERE_COLLECT_VMS', True),
-                    'vmguests': os.environ.get('VSPHERE_COLLECT_VMGUESTS', True),
-                    'datastores': os.environ.get('VSPHERE_COLLECT_DATASTORES', True),
-                    'hosts': os.environ.get('VSPHERE_COLLECT_HOSTS', True),
-                    'snapshots': os.environ.get('VSPHERE_COLLECT_SNAPSHOTS', True),
+                    'vms': get_bool_env('VSPHERE_COLLECT_VMS', True),
+                    'vmguests': get_bool_env('VSPHERE_COLLECT_VMGUESTS', True),
+                    'datastores': get_bool_env('VSPHERE_COLLECT_DATASTORES', True),
+                    'hosts': get_bool_env('VSPHERE_COLLECT_HOSTS', True),
+                    'snapshots': get_bool_env('VSPHERE_COLLECT_SNAPSHOTS', True),
                 }
             }
         }
@@ -785,13 +990,14 @@ class VMWareMetricsResource(Resource):
                 'vsphere_host': os.environ.get('VSPHERE_{}_HOST'.format(section)),
                 'vsphere_user': os.environ.get('VSPHERE_{}_USER'.format(section)),
                 'vsphere_password': os.environ.get('VSPHERE_{}_PASSWORD'.format(section)),
-                'ignore_ssl': os.environ.get('VSPHERE_{}_IGNORE_SSL'.format(section), False),
+                'ignore_ssl': get_bool_env('VSPHERE_{}_IGNORE_SSL'.format(section), False),
+                'specs_size': os.environ.get('VSPHERE_{}_SPECS_SIZE'.format(section), 5000),
                 'collect_only': {
-                    'vms': os.environ.get('VSPHERE_{}_COLLECT_VMS'.format(section), True),
-                    'vmguests': os.environ.get('VSPHERE_{}_COLLECT_VMGUESTS'.format(section), True),
-                    'datastores': os.environ.get('VSPHERE_{}_COLLECT_DATASTORES'.format(section), True),
-                    'hosts': os.environ.get('VSPHERE_{}_COLLECT_HOSTS'.format(section), True),
-                    'snapshots': os.environ.get('VSPHERE_{}_COLLECT_SNAPSHOTS'.format(section), True),
+                    'vms': get_bool_env('VSPHERE_{}_COLLECT_VMS'.format(section), True),
+                    'vmguests': get_bool_env('VSPHERE_{}_COLLECT_VMGUESTS'.format(section), True),
+                    'datastores': get_bool_env('VSPHERE_{}_COLLECT_DATASTORES'.format(section), True),
+                    'hosts': get_bool_env('VSPHERE_{}_COLLECT_HOSTS'.format(section), True),
+                    'snapshots': get_bool_env('VSPHERE_{}_COLLECT_SNAPSHOTS'.format(section), True),
                 }
             }
 
@@ -805,7 +1011,7 @@ class VMWareMetricsResource(Resource):
         try:
             yield self.generate_latest_metrics(request)
         except Exception:
-            log(traceback.format_exc())
+            logging.error(traceback.format_exc())
             request.setResponseCode(500)
             request.write(b'# Collection failed')
             request.finish()
@@ -819,7 +1025,7 @@ class VMWareMetricsResource(Resource):
         """ gets the latest metrics """
         section = request.args.get(b'section', [b'default'])[0].decode('utf-8')
         if section not in self.config.keys():
-            log("{} is not a valid section, using default".format(section))
+            logging.info("{} is not a valid section, using default".format(section))
             section = 'default'
 
         if self.config[section].get('vsphere_host') and self.config[section].get('vsphere_host') != "None":
@@ -830,7 +1036,7 @@ class VMWareMetricsResource(Resource):
             vsphere_host = request.args.get(b'vsphere_host')[0].decode('utf-8')
         else:
             request.setResponseCode(500)
-            log("No vsphere_host or target defined")
+            logging.info("No vsphere_host or target defined")
             request.write(b'No vsphere_host or target defined!\n')
             request.finish()
             return
@@ -840,6 +1046,7 @@ class VMWareMetricsResource(Resource):
             self.config[section]['vsphere_user'],
             self.config[section]['vsphere_password'],
             self.config[section]['collect_only'],
+            self.config[section]['specs_size'],
             self.config[section]['ignore_ssl'],
         )
         metrics = yield collector.collect()
@@ -861,15 +1068,37 @@ class HealthzResource(Resource):
     def render_GET(self, request):
         request.setHeader("Content-Type", "text/plain; charset=UTF-8")
         request.setResponseCode(200)
-        log("Service is UP")
+        logging.info("Service is UP")
         return 'Server is UP'.encode()
 
 
-def log(data, *args):
-    """
-    Log any message in a uniform format
-    """
-    print("[{0}] {1}".format(datetime.datetime.utcnow().replace(tzinfo=pytz.utc), data % args))
+class IndexResource(Resource):
+    isLeaf = False
+
+    def getChild(self, name, request):
+        if name == b'':
+            return self
+        return Resource.getChild(self, name, request)
+
+    def render_GET(self, request):
+        output = """<html>
+            <head><title>VMware Exporter</title></head>
+            <body>
+            <h1>VMware Exporter</h1>
+            <p><a href="/metrics">Metrics</a></p>
+            </body>
+            </html>"""
+        request.setHeader("Content-Type", "text/html; charset=UTF-8")
+        request.setResponseCode(200)
+        return output.encode()
+
+
+def registerEndpoints(args):
+    root = Resource()
+    root.putChild(b'', IndexResource())
+    root.putChild(b'metrics', VMWareMetricsResource(args))
+    root.putChild(b'healthz', HealthzResource())
+    return root
 
 
 def main(argv=None):
@@ -879,18 +1108,20 @@ def main(argv=None):
                         default=None, help="configuration file")
     parser.add_argument('-p', '--port', dest='port', type=int,
                         default=9272, help="HTTP port to expose metrics")
+    parser.add_argument('-l', '--loglevel', dest='loglevel',
+                        default="INFO", help="Set application loglevel INFO, DEBUG")
 
     args = parser.parse_args(argv or sys.argv[1:])
 
+    numeric_level = getattr(logging, args.loglevel.upper(), None)
+    if not isinstance(numeric_level, int):
+        raise ValueError("Invalid log level: {level}".format(level=args.loglevel))
+    logging.basicConfig(level=numeric_level, format='%(asctime)s %(levelname)s:%(message)s')
+
     reactor.suggestThreadPoolSize(25)
 
-    # Start up the server to expose the metrics.
-    root = Resource()
-    root.putChild(b'metrics', VMWareMetricsResource(args))
-    root.putChild(b'healthz', HealthzResource())
-
-    factory = Site(root)
-    log("Starting web server on port {}".format(args.port))
+    factory = Site(registerEndpoints(args))
+    logging.info("Starting web server on port {port}".format(port=args.port))
     endpoint = endpoints.TCP4ServerEndpoint(reactor, args.port)
     endpoint.listen(factory)
     reactor.run()
